@@ -1,4 +1,6 @@
-// This file is part of Projecteur - https://github.com/jahnf/projecteur - See LICENSE.md and README.md
+// This file is part of Projecteur - https://github.com/jahnf/projecteur
+// - See LICENSE.md and README.md
+
 #include "inputmapconfig.h"
 
 #include "actiondelegate.h"
@@ -11,16 +13,12 @@
 // -------------------------------------------------------------------------------------------------
 namespace  {
   const InputMapModelItem invalidItem_;
-}
+} // end anonymous namespace
 
 // -------------------------------------------------------------------------------------------------
-InputMapConfigModel::InputMapConfigModel(QObject* parent)
-  : InputMapConfigModel(nullptr, parent)
-{}
-
-// -------------------------------------------------------------------------------------------------
-InputMapConfigModel::InputMapConfigModel(InputMapper* im, QObject* parent)
+InputMapConfigModel::InputMapConfigModel(InputMapper* im, const DeviceId& dId, QObject* parent)
   : QAbstractTableModel(parent)
+  , m_currentDeviceId(dId)
   , m_inputMapper(im)
 {}
 
@@ -39,10 +37,9 @@ int InputMapConfigModel::columnCount(const QModelIndex& /*parent*/) const
 // -------------------------------------------------------------------------------------------------
 Qt::ItemFlags InputMapConfigModel::flags(const QModelIndex &index) const
 {
-  if (index.column() == InputSeqCol)
-    return (QAbstractTableModel::flags(index) | Qt::ItemIsEditable);
-  else if (index.column() == ActionCol)
+  if (index.column() == InputSeqCol || index.column() == ActionCol) {
     return (QAbstractTableModel::flags(index) | Qt::ItemIsEditable);
+  }
 
   return QAbstractTableModel::flags(index) & ~Qt::ItemIsEditable;
 }
@@ -66,6 +63,7 @@ QVariant InputMapConfigModel::headerData(int section, Qt::Orientation orientatio
     case InputSeqCol: return tr("Input Sequence");
     case ActionTypeCol: return "Type";
     case ActionCol: return tr("Mapped Action");
+    default: return "Invalid";
     }
   }
   else if (orientation == Qt::Vertical)
@@ -83,8 +81,9 @@ QVariant InputMapConfigModel::headerData(int section, Qt::Orientation orientatio
 // -------------------------------------------------------------------------------------------------
 const InputMapModelItem& InputMapConfigModel::configData(const QModelIndex& index) const
 {
-  if (index.row() >= static_cast<int>(m_configItems.size()))
+  if (index.row() >= static_cast<int>(m_configItems.size())) {
     return invalidItem_;
+  }
 
   return m_configItems[index.row()];
 }
@@ -92,7 +91,7 @@ const InputMapModelItem& InputMapConfigModel::configData(const QModelIndex& inde
 // -------------------------------------------------------------------------------------------------
 void InputMapConfigModel::removeConfigItemRows(int fromRow, int toRow)
 {
-  if (fromRow > toRow) return;
+  if (fromRow > toRow) { return; }
 
   beginRemoveRows(QModelIndex(), fromRow, toRow);
   for (int i = toRow; i >= fromRow && i < m_configItems.size(); --i) {
@@ -105,7 +104,7 @@ void InputMapConfigModel::removeConfigItemRows(int fromRow, int toRow)
 // -------------------------------------------------------------------------------------------------
 int InputMapConfigModel::addNewItem(std::shared_ptr<Action> action)
 {
-  if (!action) return -1;
+  if (!action) { return -1; }
 
   const auto row = m_configItems.size();
   beginInsertRows(QModelIndex(), row, row);
@@ -126,7 +125,7 @@ void InputMapConfigModel::configureInputMapper()
 // -------------------------------------------------------------------------------------------------
 void InputMapConfigModel::removeConfigItemRows(std::vector<int> rows)
 {
-  if (rows.empty()) return;
+  if (rows.empty()) { return; }
   std::sort(rows.rbegin(), rows.rend());
 
   int seq_last = rows.front();
@@ -161,6 +160,21 @@ void InputMapConfigModel::setInputSequence(const QModelIndex& index, const KeyEv
       --m_duplicates[c.deviceSequence];
       ++m_duplicates[kes];
       c.deviceSequence = kes;
+
+      const bool isSpecialMoveInput = !SpecialKeys::logitechSpotlightHoldMove(c.deviceSequence).name.isEmpty();
+
+      const bool isMoveAction =
+        (c.action->type() == Action::Type::ScrollHorizontal
+        || c.action->type() == Action::Type::ScrollVertical
+        || c.action->type() == Action::Type::VolumeControl);
+
+      if (!isSpecialMoveInput && isMoveAction) {
+        setItemActionType(index, Action::Type::KeySequence);
+      }
+      else if (isSpecialMoveInput && !isMoveAction) {
+        setItemActionType(index, Action::Type::ScrollVertical);
+      }
+
       configureInputMapper();
       updateDuplicates();
       emit dataChanged(index, index, {Qt::DisplayRole, Roles::InputSeqRole});
@@ -174,7 +188,8 @@ void InputMapConfigModel::setKeySequence(const QModelIndex& index, const NativeK
   if (index.row() < static_cast<int>(m_configItems.size()))
   {
     auto& c = m_configItems[index.row()];
-    // TODO if action is currently not a keysequence action.. -> just change action type?
+    // If the current action is not a keysequence action
+    // -> setting the key sequence is currently ignored.
     if (auto action = std::dynamic_pointer_cast<KeySequenceAction>(c.action))
     {
       if (action->keySequence != ks) {
@@ -189,9 +204,9 @@ void InputMapConfigModel::setKeySequence(const QModelIndex& index, const NativeK
 // -------------------------------------------------------------------------------------------------
 void InputMapConfigModel::setItemActionType(const QModelIndex& idx, Action::Type type)
 {
-  if (idx.row() >= m_configItems.size()) return;
+  if (idx.row() >= m_configItems.size()) { return; }
   auto& item = m_configItems[idx.row()];
-  if (item.action->type() == type) return;
+  if (item.action->type() == type) { return; }
 
   switch(type)
   {
@@ -204,6 +219,15 @@ void InputMapConfigModel::setItemActionType(const QModelIndex& idx, Action::Type
   case Action::Type::ToggleSpotlight:
     item.action = std::make_shared<ToggleSpotlightAction>();
     break;
+  case Action::Type::ScrollHorizontal:
+    item.action = GlobalActions::scrollHorizontal();
+    break;
+  case Action::Type::ScrollVertical:
+    item.action = GlobalActions::scrollVertical();
+    break;
+  case Action::Type::VolumeControl:
+    item.action = GlobalActions::volumeControl();
+    break;
   }
 
   configureInputMapper();
@@ -233,7 +257,7 @@ InputMapConfig InputMapConfigModel::configuration() const
 
   for (const auto& item : m_configItems)
   {
-    if (item.deviceSequence.size() == 0) continue;
+    if (item.deviceSequence.empty()) { continue; }
     config.emplace(item.deviceSequence, MappedAction{item.action});
   }
 
@@ -256,6 +280,18 @@ void InputMapConfigModel::setConfiguration(const InputMapConfig& config)
   endResetModel();
 }
 
+// -------------------------------------------------------------------------------------------------
+const DeviceId& InputMapConfigModel::deviceId() const
+{
+  return m_currentDeviceId;
+}
+
+// -------------------------------------------------------------------------------------------------
+void InputMapConfigModel::setDeviceId(const DeviceId& dId)
+{
+  m_currentDeviceId = dId;
+}
+
 // -------------------------------------------------------------------------------------------------
 void InputMapConfigModel::updateDuplicates()
 {
@@ -271,12 +307,11 @@ void InputMapConfigModel::updateDuplicates()
   }
 }
 
-#include <QMenu>
 // -------------------------------------------------------------------------------------------------
 // -------------------------------------------------------------------------------------------------
 InputMapConfigView::InputMapConfigView(QWidget* parent)
-  : QTableView(parent)
-    , m_actionTypeDelegate(new ActionTypeDelegate(this))
+  : QTableView(parent),
+    m_actionTypeDelegate(new ActionTypeDelegate(this))
 {
   verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
 
@@ -296,26 +331,30 @@ InputMapConfigView::InputMapConfigView(QWidget* parent)
   setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
 
   connect(this, &QWidget::customContextMenuRequested, this,
-  [this, actionDelegate](const QPoint& pos)
+  [this, imSeqDelegate, actionDelegate](const QPoint& pos)
   {
     const auto idx = indexAt(pos);
-    if (!idx.isValid()) return;
+    if (!idx.isValid()) { return; }
 
-    if (idx.column() == InputMapConfigModel::ActionCol)
+    switch(idx.column())
     {
-      actionDelegate->actionContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
-                                        idx, this->viewport()->mapToGlobal(pos));
-    }
-    else if (idx.column() == InputMapConfigModel::ActionTypeCol)
-    {
-      m_actionTypeDelegate->actionContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
-                                              idx, this->viewport()->mapToGlobal(pos));
-    }
+      case InputMapConfigModel::InputSeqCol:
+        imSeqDelegate->inputSeqContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
+                                                  idx, this->viewport()->mapToGlobal(pos));
+        break;
+      case InputMapConfigModel::ActionTypeCol:
+        m_actionTypeDelegate->actionContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
+                                                idx, this->viewport()->mapToGlobal(pos));
+        break;
+      case InputMapConfigModel::ActionCol:
+        actionDelegate->actionContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
+                                          idx, this->viewport()->mapToGlobal(pos));
+    };
   });
 
   connect(this, &QTableView::doubleClicked, this, [this](const QModelIndex& idx)
   {
-    if (!idx.isValid()) return;
+    if (!idx.isValid()) { return; }
     if (idx.column() == InputMapConfigModel::ActionTypeCol) {
       const auto pos = viewport()->mapToGlobal(visualRect(currentIndex()).bottomLeft());
       m_actionTypeDelegate->actionContextMenu(this, qobject_cast<InputMapConfigModel*>(model()),
@@ -354,15 +393,16 @@ void InputMapConfigView::keyPressEvent(QKeyEvent* e)
     }
     break;
   case Qt::Key_Delete:
-    if (const auto imModel = qobject_cast<InputMapConfigModel*>(model()))
-    switch (currentIndex().column())
-    {
-    case InputMapConfigModel::InputSeqCol:
-      imModel->setInputSequence(currentIndex(), KeyEventSequence{});
-      return;
-    case InputMapConfigModel::ActionCol:
-      imModel->setKeySequence(currentIndex(), NativeKeySequence());
-      return;
+    if (const auto imModel = qobject_cast<InputMapConfigModel*>(model())) {
+      switch (currentIndex().column())
+      {
+      case InputMapConfigModel::InputSeqCol:
+        imModel->setInputSequence(currentIndex(), KeyEventSequence{});
+        return;
+      case InputMapConfigModel::ActionCol:
+        imModel->setKeySequence(currentIndex(), NativeKeySequence());
+        return;
+      }
     }
     break;
   case Qt::Key_Tab:
