@@ -1,4 +1,6 @@
-// This file is part of Projecteur - https://github.com/jahnf/projecteur - See LICENSE.md and README.md
+// This file is part of Projecteur - https://github.com/jahnf/projecteur
+// - See LICENSE.md and README.md
+
 #include "preferencesdlg.h"
 
 #include "projecteur-GitVersion.h"  // auto generated version information
@@ -9,9 +11,9 @@
 #include "logging.h"
 #include "settings.h"
 
+#include <QCheckBox>
 #include <QComboBox>
 #include <QCoreApplication>
-#include <QCheckBox>
 #include <QDateTime>
 #include <QDoubleSpinBox>
 #include <QFileDialog>
@@ -29,8 +31,10 @@
 #include <QTabWidget>
 #include <QTimer>
 
-#if HAS_Qt5_X11Extras
-#include <QX11Info>
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+  #if HAS_Qt_X11Extras
+  #include <QX11Info>
+  #endif
 #endif
 
 #include <map>
@@ -51,7 +55,7 @@ namespace {
     { CURSOR_PATH "cursor-uparrow.png", {"Up Arrow Cursor", Qt::UpArrowCursor}},
     { CURSOR_PATH "cursor-whatsthis.png", {"What't This Cursor", Qt::WhatsThisCursor}},
   };
-}
+} // end anonymous namespace
 
 // -------------------------------------------------------------------------------------------------
 PreferencesDialog::PreferencesDialog(Settings* settings, Spotlight* spotlight,
@@ -79,7 +83,8 @@ PreferencesDialog::PreferencesDialog(Settings* settings, Spotlight* spotlight,
 
   const auto tabWidget = new QTabWidget(this);
   tabWidget->addTab(settingsWidget, tr("Spotlight"));
-  tabWidget->addTab(new DevicesWidget(settings, spotlight, this), tr("Devices"));
+  m_deviceswidget = new DevicesWidget(settings, spotlight, this);
+  tabWidget->addTab(m_deviceswidget, tr("Devices"));
   tabWidget->addTab(createLogTabWidget(), tr("Log"));
 
   const auto overlayCheckBox = new QCheckBox(this);
@@ -97,7 +102,6 @@ PreferencesDialog::PreferencesDialog(Settings* settings, Spotlight* spotlight,
 
   connect(overlayCheckBox, &QCheckBox::toggled, this, [settings](bool checked){
     settings->setOverlayDisabled(!checked);
-
   });
 
   connect(settings, &Settings::overlayDisabledChanged, this,
@@ -145,7 +149,7 @@ QWidget* PreferencesDialog::createSettingsTabWidget(Settings* settings)
   const auto mainVBox = new QVBoxLayout(widget);
   mainVBox->addLayout(mainHBox);
   mainVBox->addWidget(presetSelector);
-#if HAS_Qt5_X11Extras
+#if HAS_Qt_X11Extras
   mainVBox->addWidget(createCompositorWarningWidget());
 #endif
   mainVBox->addLayout(hbox);
@@ -188,8 +192,9 @@ QWidget* PreferencesDialog::createPresetSelector(Settings* settings)
     deleteBtn->setEnabled(index > 0);
     m_presetCombo->setStyle(index == 0 ? &*m_presetComboStyle : normalComboStyle);
 
-    if (index > 0 && !m_presetCombo->currentText().isEmpty())
+    if (index > 0 && !m_presetCombo->currentText().isEmpty()) {
       settings->loadPreset(m_presetCombo->currentText());
+    }
   });
 
   connect(newBtn, &QPushButton::clicked, this, [newBtn, settings, this]()
@@ -230,7 +235,7 @@ QWidget* PreferencesDialog::createPresetSelector(Settings* settings)
 
   connect(deleteBtn, &QPushButton::clicked, this, [this, settings]()
   {
-    if (m_presetCombo->currentIndex() < 0) return;
+    if (m_presetCombo->currentIndex() < 0) { return; }
     settings->removePreset(m_presetCombo->currentText());
   });
 
@@ -252,7 +257,7 @@ QWidget* PreferencesDialog::createPresetSelector(Settings* settings)
 }
 
 // -------------------------------------------------------------------------------------------------
-#if HAS_Qt5_X11Extras
+#if HAS_Qt_X11Extras
 QWidget* PreferencesDialog::createCompositorWarningWidget()
 {
   if (!QX11Info::isPlatformX11())
@@ -365,7 +370,7 @@ QGroupBox* PreferencesDialog::createShapeGroupBox(Settings* settings)
   // Function for updating all spotlight shape related widgets
   auto updateShapeSettingsWidgets = [settings, shapeCombo, shapeRotationSb, shapeRotationLabel, spotGrid, this]()
   {
-    if (shapeCombo->currentIndex() == -1) return;
+    if (shapeCombo->currentIndex() == -1) { return; }
     const QString shapeQml = shapeCombo->itemData(shapeCombo->currentIndex()).toString();
     const auto& shapes = settings->spotShapes();
     auto it = std::find_if(shapes.cbegin(), shapes.cend(), [&shapeQml](const Settings::SpotShape& s) {
@@ -398,9 +403,13 @@ QGroupBox* PreferencesDialog::createShapeGroupBox(Settings* settings)
       int row = startRow;
       for (const auto& s : it->shapeSettings())
       {
-        if (row >= startRow + maxRows) break;
+        if (row >= startRow + maxRows) { break; }
         spotGrid->addWidget(new QLabel(s.displayName(), this),row, 0);
+        #if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
         if (s.defaultValue().type() == QVariant::Int)
+        #else
+        if (s.defaultValue().metaType().id() == QMetaType::Int)
+        #endif
         {
           const auto spinbox = new QSpinBox(this);
           spinbox->setMaximum(s.maxValue().toInt());
@@ -419,7 +428,7 @@ QGroupBox* PreferencesDialog::createShapeGroupBox(Settings* settings)
             connect(pm, &QQmlPropertyMap::valueChanged, spinbox,
             [s, spinbox, this](const QString& key, const QVariant& value)
             {
-              if (key != s.settingsKey() || !value.isValid()) return;
+              if (key != s.settingsKey() || !value.isValid()) { return; }
               spinbox->setValue(value.toInt());
               resetPresetCombo();
             });
@@ -723,7 +732,7 @@ QWidget* PreferencesDialog::createLogTabWidget()
     QString logFilter(tr("Log files (*.log *.txt)"));
     const auto logFile = QFileDialog::getSaveFileName(this, tr("Save log file"),
                                                       defaultFile, logFilter, &logFilter);
-    if (logFile.isEmpty())  return;
+    if (logFile.isEmpty())  { return; }
     saveDir = QFileInfo(logFile).path();
 
     QFile f(logFile);
@@ -761,8 +770,9 @@ QWidget* PreferencesDialog::createLogTabWidget()
 // -------------------------------------------------------------------------------------------------
 void PreferencesDialog::setMode(Mode dialogMode)
 {
-  if (m_dialogMode == dialogMode)
+  if (m_dialogMode == dialogMode) {
     return;
+  }
 
   setDialogMode(dialogMode);
 }
@@ -792,14 +802,15 @@ void PreferencesDialog::setDialogMode(Mode dialogMode)
 // -------------------------------------------------------------------------------------------------
 void PreferencesDialog::resetPresetCombo()
 {
-  if (m_presetCombo) m_presetCombo->setCurrentIndex(0);
+  if (m_presetCombo) { m_presetCombo->setCurrentIndex(0); }
 }
 
 // -------------------------------------------------------------------------------------------------
 void PreferencesDialog::setDialogActive(bool active)
 {
-  if (active == m_active)
+  if (active == m_active) {
     return;
+  }
 
   m_active = active;
   emit dialogActiveChanged(active);
@@ -818,7 +829,7 @@ bool PreferencesDialog::event(QEvent* e)
 }
 
 // -------------------------------------------------------------------------------------------------
-void PreferencesDialog::closeEvent(QCloseEvent*)
+void PreferencesDialog::closeEvent(QCloseEvent* /* ev */)
 {
   if (m_dialogMode == Mode::MinimizeOnlyDialog) {
     emit exitApplicationRequested();
