@@ -1,13 +1,17 @@
-// This file is part of Projecteur - https://github.com/jahnf/projecteur - See LICENSE.md and README.md
+// This file is part of Projecteur - https://github.com/jahnf/projecteur
+// - See LICENSE.md and README.md
 #pragma once
-#include "spotlight.h"
+
+#include "devicescan.h"
 
 #include <QApplication>
+#include <QPointer>
 
 #include <map>
 #include <memory>
 
 class AboutDialog;
+class DeviceCommandHelper;
 class LinuxDesktop;
 class PreferencesDialog;
 class QLocalServer;
@@ -17,7 +21,7 @@ class QQmlApplicationEngine;
 class QQmlComponent;
 class QSystemTrayIcon;
 class Settings;
-class Settings;
+class Spotlight;
 
 class ProjecteurApplication : public QApplication
 {
@@ -67,14 +71,18 @@ private:
   QPoint currentCursorPos() const;
   void setCurrentCursorPos(const QPoint& pos);
 
+  void setupTrayIcon();
+  void setupSpotlight();
+
 private:
   std::unique_ptr<QSystemTrayIcon> m_trayIcon;
   std::unique_ptr<QMenu> m_trayMenu;
   std::unique_ptr<PreferencesDialog> m_dialog;
-  std::unique_ptr<AboutDialog> m_aboutDialog;
+  QPointer<AboutDialog> m_aboutDialog;
   QLocalServer* const m_localServer = nullptr;
-  Spotlight* m_spotlight = nullptr;
   Settings* m_settings = nullptr;
+  Spotlight* m_spotlight = nullptr;
+  DeviceCommandHelper* m_deviceCommandHelper = nullptr;
   LinuxDesktop* m_linuxDesktop = nullptr;
   QQmlApplicationEngine* m_qmlEngine = nullptr;
   QQmlComponent* m_windowQmlComponent = nullptr;
