@@ -1,6 +1,9 @@
-// This file is part of Projecteur - https://github.com/jahnf/projecteur - See LICENSE.md and README.md
+// This file is part of Projecteur - https://github.com/jahnf/projecteur
+// - See LICENSE.md and README.md
+
 #include "spotlight.h"
 
+#include "device-hidpp.h"
 #include "deviceinput.h"
 #include "logging.h"
 #include "settings.h"
@@ -10,6 +13,7 @@
 #include <QTimer>
 #include <QVarLengthArray>
 
+#include <cmath>
 #include <fcntl.h>
 #include <sys/inotify.h>
 #include <sys/ioctl.h>
@@ -17,10 +21,54 @@
 
 DECLARE_LOGGING_CATEGORY(device)
 DECLARE_LOGGING_CATEGORY(hid)
+DECLARE_LOGGING_CATEGORY(input)
 
 namespace {
   const auto hexId = logging::hexId;
-} // --- end anonymous namespace
+
+  // See details on workaround in onEventDataAvailable
+  bool workaroundLogitechFirstMoveEvent = true;
+
+} // end anonymous namespace
+
+
+// -------------------------------------------------------------------------------------------------
+// Hold button state. Very much Logitech Spotlight specific.
+struct HoldButtonStatus
+{
+  void setButtonsPressed(bool nextPressed, bool backPressed)
+  {
+    if (!m_nextPressed && nextPressed) {
+      m_moveKeyEvSeq = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::NextHoldMove).keyEventSeq;
+    } else if (!m_backPressed && backPressed) {
+      m_moveKeyEvSeq = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::BackHoldMove).keyEventSeq;
+    } else if (m_nextPressed && !nextPressed && backPressed) {
+      m_moveKeyEvSeq = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::BackHoldMove).keyEventSeq;
+    } else if (m_backPressed && !backPressed && nextPressed) {
+      m_moveKeyEvSeq = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::NextHoldMove).keyEventSeq;
+    }
+
+    m_nextPressed = nextPressed;
+    m_backPressed = backPressed;
+
+    if (!nextPressed && !backPressed) { m_moveKeyEvSeq.clear(); }
+  }
+
+  bool nextPressed() const { return m_nextPressed; }
+  bool backPressed() const { return m_backPressed; }
+
+  void reset() { m_nextPressed = m_backPressed = false; m_moveKeyEvSeq.clear(); };
+
+  const KeyEventSequence& moveKeyEventSeq() const {
+    return m_moveKeyEvSeq;
+  };
+
+private:
+  bool m_nextPressed = false;
+  bool m_backPressed = false;
+
+  KeyEventSequence m_moveKeyEvSeq;
+};
 
 // -------------------------------------------------------------------------------------------------
 Spotlight::Spotlight(QObject* parent, Options options, Settings* settings)
@@ -28,32 +76,44 @@ Spotlight::Spotlight(QObject* parent, Options options, Settings* settings)
   , m_options(std::move(options))
   , m_activeTimer(new QTimer(this))
   , m_connectionTimer(new QTimer(this))
+  , m_holdMoveEventTimer(new QTimer(this))
   , m_settings(settings)
+  , m_holdButtonStatus(std::make_unique<HoldButtonStatus>())
 {
+  constexpr int spotlightActiveTimoutMs = 600;
   m_activeTimer->setSingleShot(true);
-  m_activeTimer->setInterval(600);
+  m_activeTimer->setInterval(spotlightActiveTimoutMs);
 
   connect(m_activeTimer, &QTimer::timeout, this, [this](){
     setSpotActive(false);
+    workaroundLogitechFirstMoveEvent = true;
   });
 
   if (m_options.enableUInput) {
-    m_virtualDevice = VirtualDevice::create();
+    m_virtualMouseDevice = VirtualDevice::create(
+      VirtualDevice::Type::Mouse, "Projecteur_virtual_mouse");
+    m_virtualKeyDevice = VirtualDevice::create(
+      VirtualDevice::Type::Keyboard, "Projecteur_virtual_keyboard");
   }
   else {
     logInfo(device) << tr("Virtual device initialization was skipped.");
   }
 
   m_connectionTimer->setSingleShot(true);
-  // From detecting a change from inotify, the device needs some time to be ready for open
+  // From detecting a change with inotify, the device needs some time to be ready for open,
+  // otherwise opening the device will fail.
   // TODO: This interval seems to work, but it is arbitrary - there should be a better way.
-  m_connectionTimer->setInterval(800);
+  constexpr int delayedConnectionTimerIntervalMs = 800;
+  m_connectionTimer->setInterval(delayedConnectionTimerIntervalMs);
 
   connect(m_connectionTimer, &QTimer::timeout, this, [this]() {
     logDebug(device) << tr("New connection check triggered");
     connectDevices();
   });
 
+  m_holdMoveEventTimer->setSingleShot(true);
+  m_holdMoveEventTimer->setInterval(30);
+
   // Try to find already attached device(s) and connect to it.
   connectDevices();
   setupDevEventInotify();
@@ -66,7 +126,7 @@ Spotlight::~Spotlight() = default;
 bool Spotlight::anySpotlightDeviceConnected() const
 {
   for (const auto& dc : m_deviceConnections) {
-    if (dc.second->subDeviceCount()) return true;
+    if (dc.second->subDeviceCount()) { return true; }
   }
   return false;
 }
@@ -76,7 +136,7 @@ uint32_t Spotlight::connectedDeviceCount() const
 {
   uint32_t count = 0;
   for (const auto& dc : m_deviceConnections) {
-    if (dc.second->subDeviceCount()) ++count;
+    if (dc.second->subDeviceCount()) { ++count; }
   }
   return count;
 }
@@ -84,9 +144,9 @@ uint32_t Spotlight::connectedDeviceCount() const
 // -------------------------------------------------------------------------------------------------
 void Spotlight::setSpotActive(bool active)
 {
-  if (m_spotActive == active) return;
+  if (m_spotActive == active) { return; }
   m_spotActive = active;
-  if (!m_spotActive) m_activeTimer->stop();
+  if (!m_spotActive) { m_activeTimer->stop(); }
   emit spotActiveChanged(m_spotActive);
 }
 
@@ -117,28 +177,81 @@ int Spotlight::connectDevices()
   {
     auto& dc = m_deviceConnections[dev.id];
     if (!dc) {
-      dc = std::make_shared<DeviceConnection>(dev.id, dev.getName(), m_virtualDevice);
+      dc = std::make_shared<DeviceConnection>(
+        dev.id, dev.getName(), m_virtualMouseDevice, m_virtualKeyDevice);
     }
 
     const bool anyConnectedBefore = anySpotlightDeviceConnected();
     for (const auto& scanSubDevice : dev.subDevices)
     {
-      if (!scanSubDevice.deviceReadable) continue;
-      if (dc->hasSubDevice(scanSubDevice.deviceFile)) continue;
+      if (!scanSubDevice.deviceReadable)
+      {
+        logWarn(device) << tr("Sub-device not readable: %1 (%2:%3) %4")
+          .arg(dc->deviceName(), hexId(dev.id.vendorId), hexId(dev.id.productId), scanSubDevice.deviceFile);
+        continue;
+      }
+      if (dc->hasSubDevice(scanSubDevice.deviceFile)) { continue; }
 
       std::shared_ptr<SubDeviceConnection> subDeviceConnection =
-      [&scanSubDevice, &dc, this]() -> std::shared_ptr<SubDeviceConnection> {
+      [&scanSubDevice, &dc, this]() -> std::shared_ptr<SubDeviceConnection>
+      { // Input event sub devices
         if (scanSubDevice.type == DeviceScan::SubDevice::Type::Event) {
           auto devCon = SubEventConnection::create(scanSubDevice, *dc);
-          if (addInputEventHandler(devCon)) return devCon;
-        } else if (scanSubDevice.type == DeviceScan::SubDevice::Type::Hidraw) {
-          auto hidCon = SubHidrawConnection::create(scanSubDevice, *dc);
-          if(addHIDInputHandler(hidCon)) return hidCon;
+          if (addInputEventHandler(devCon)) { return devCon; }
+        } // Hidraw sub devices
+        else if (scanSubDevice.type == DeviceScan::SubDevice::Type::Hidraw)
+        {
+          if (dc->hasHidppSupport())
+          {
+            if (auto hidppCon = SubHidppConnection::create(scanSubDevice, *dc))
+            {
+              QPointer<SubHidppConnection> connPtr(hidppCon.get());
+
+              connect(&*hidppCon, &SubHidppConnection::featureSetInitialized, this,
+              [this, connPtr](){
+                if (!connPtr) { return; }
+                this->registerForNotifications(connPtr.data());
+              });
+
+              // Remove device on socketReadError
+              connect(&*hidppCon, &SubHidppConnection::socketReadError, this, [this, connPtr](){
+                if (!connPtr) { return; }
+                const bool anyConnectedBefore = anySpotlightDeviceConnected();
+                connPtr->disconnect();
+                QTimer::singleShot(0, this, [this, devicePath=connPtr->path(), anyConnectedBefore](){
+                  removeDeviceConnection(devicePath);
+                  if (!anySpotlightDeviceConnected() && anyConnectedBefore) {
+                    emit anySpotlightDeviceConnectedChanged(false);
+                  }
+                });
+              });
+
+              return hidppCon;
+            }
+          }
+          else if (auto hidrawConn = SubHidrawConnection::create(scanSubDevice, *dc))
+          {
+            QPointer<SubHidrawConnection> connPtr(hidrawConn.get());
+            // Remove device on socketReadError
+            connect(&*hidrawConn, &SubHidrawConnection::socketReadError, this, [this, connPtr](){
+              if (!connPtr) { return; }
+              const bool anyConnectedBefore = anySpotlightDeviceConnected();
+              connPtr->disconnect();
+              QTimer::singleShot(0, this, [this, devicePath=connPtr->path(), anyConnectedBefore](){
+                removeDeviceConnection(devicePath);
+                if (!anySpotlightDeviceConnected() && anyConnectedBefore) {
+                  emit anySpotlightDeviceConnectedChanged(false);
+                }
+              });
+            });
+
+            return hidrawConn;
+          }
         }
         return std::shared_ptr<SubDeviceConnection>();
       }();
 
-      if (!subDeviceConnection) continue;
+      if (!subDeviceConnection) { continue; }
 
       if (dc->subDeviceCount() == 0) {
         // Load Input mapping settings when first sub-device gets added.
@@ -153,7 +266,7 @@ int Spotlight::connectDevices()
 
         static QString lastPreset;
 
-        connect(im, &InputMapper::actionMapped, this, [this](std::shared_ptr<Action> action)
+        connect(im, &InputMapper::actionMapped, this, [this](const std::shared_ptr<Action>& action)
         {
           if (action->type() == Action::Type::CyclePresets)
           {
@@ -172,6 +285,31 @@ int Spotlight::connectDevices()
           {
             m_settings->setOverlayDisabled(!m_settings->overlayDisabled());
           }
+          else if (action->type() == Action::Type::ScrollHorizontal || action->type() == Action::Type::ScrollVertical)
+          {
+            if (!m_virtualMouseDevice) { return; }
+
+            const int param = (action->type() == Action::Type::ScrollHorizontal)
+              ? static_cast<ScrollHorizontalAction*>(action.get())->param
+              : static_cast<ScrollVerticalAction*>(action.get())->param;
+
+            if (param)
+            {
+              const uint16_t wheelCode = (action->type() == Action::Type::ScrollHorizontal) ? REL_HWHEEL : REL_WHEEL;
+              const std::vector<input_event> scrollInputEvents = {{{}, EV_REL, wheelCode, param}, {{}, EV_SYN, SYN_REPORT, 0},};
+              m_virtualMouseDevice->emitEvents(scrollInputEvents);
+            }
+          }
+          else if (action->type() == Action::Type::VolumeControl)
+          {
+            if (!m_virtualMouseDevice) { return; }
+
+            auto param = static_cast<VolumeControlAction*>(action.get())->param;
+            uint16_t keyCode = (param > 0)? KEY_VOLUMEUP: KEY_VOLUMEDOWN;
+            const std::vector<input_event> curVolInputEvents = {{{}, EV_KEY, keyCode, 1}, {{}, EV_SYN, SYN_REPORT, 0},
+                                                                {{}, EV_KEY, keyCode, 0}, {{}, EV_SYN, SYN_REPORT, 0},};
+            if (param) { m_virtualMouseDevice->emitEvents(curVolInputEvents); }
+          }
         });
 
         connect(m_settings, &Settings::presetLoaded, this, [](const QString& preset){
@@ -187,7 +325,7 @@ int Spotlight::connectDevices()
           logInfo(device) << tr("Connected device: %1 (%2:%3)")
                              .arg(devName, hexId(id.vendorId), hexId(id.productId));
           emit deviceConnected(id, devName);
-          if (!anyConnectedBefore) emit anySpotlightDeviceConnectedChanged(true);
+          if (!anyConnectedBefore) { emit anySpotlightDeviceConnectedChanged(true); }
         });
       }
 
@@ -236,7 +374,7 @@ void Spotlight::removeDeviceConnection(const QString &devicePath)
 // -------------------------------------------------------------------------------------------------
 void Spotlight::onEventDataAvailable(int fd, SubEventConnection& connection)
 {
-  const bool isNonBlocking = !!(connection.flags() & DeviceFlag::NonBlocking);
+  const bool isNonBlocking = connection.hasFlags(DeviceFlag::NonBlocking);
   while (true)
   {
     auto& buf = connection.inputBuffer();
@@ -246,7 +384,7 @@ void Spotlight::onEventDataAvailable(int fd, SubEventConnection& connection)
       if (errno != EAGAIN)
       {
         const bool anyConnectedBefore = anySpotlightDeviceConnected();
-        connection.disable();
+        connection.disconnect();
         QTimer::singleShot(0, this, [this, devicePath=connection.path(), anyConnectedBefore](){
           removeDeviceConnection(devicePath);
           if (!anySpotlightDeviceConnected() && anyConnectedBefore) {
@@ -264,13 +402,35 @@ void Spotlight::onEventDataAvailable(int fd, SubEventConnection& connection)
       const auto &first_ev = buf[0];
       const bool isMouseMoveEvent = first_ev.type == EV_REL
                                     && (first_ev.code == REL_X || first_ev.code == REL_Y);
+
       if (isMouseMoveEvent)
       { // Skip input mapping for mouse move events completely
-        if (!m_activeTimer->isActive()) {
+
+        // Note: During a Next or Back button press the Logitech Spotlight device can send
+        // move events via hid++ notifications. It seems that just when releasing the
+        // next or back button sometimes a mouse move event 'leaks' through here as
+        // relative input event causing the spotlight to be activated.
+        // The workaround skips a first input move event from the logitech spotlight device.
+        const bool isLogitechSpotlight = connection.deviceId().vendorId == 0x46d
+          && (connection.deviceId().productId == 0xc53e || connection.deviceId().productId == 0xb503);
+        const bool logitechIsFirst = isLogitechSpotlight && workaroundLogitechFirstMoveEvent;
+
+        if (isLogitechSpotlight)
+        {
+          workaroundLogitechFirstMoveEvent = false;
+          if(!logitechIsFirst) {
+            if (!spotActive()) { setSpotActive(true); }
+          }
+        }
+        else if (!m_activeTimer->isActive()) {
           setSpotActive(true);
         }
+
         m_activeTimer->start();
-        if (m_virtualDevice) m_virtualDevice->emitEvents(buf.data(), buf.pos());
+        if (m_virtualMouseDevice) {
+          // forward events to virtual mouse device
+          m_virtualMouseDevice->emitEvents(buf.data(), buf.pos());
+        }
       }
       else
       { // Forward events to input mapper for the device
@@ -285,31 +445,102 @@ void Spotlight::onEventDataAvailable(int fd, SubEventConnection& connection)
       buf.reset();
     }
 
-    if (!isNonBlocking) break;
+    if (!isNonBlocking) { break; }
   } // end while loop
 }
 
 // -------------------------------------------------------------------------------------------------
-void Spotlight::onHIDDataAvailable(int fd, SubHidrawConnection& connection)
+void Spotlight::registerForNotifications(SubHidppConnection* connection)
 {
-  QByteArray readVal(20, 0);
-  if (::read(fd, static_cast<void *>(readVal.data()), readVal.length()) < 0)
+  using namespace HIDPP;
+
+  // Logitech button next and back press and hold + movement
+  if (const auto rcIndex = connection->featureSet().featureIndex(FeatureCode::ReprogramControlsV4))
   {
-    if (errno != EAGAIN)
+    connection->registerNotificationCallback(this, rcIndex, makeSafeCallback(
+    [this, connection](Message&& msg)
     {
-      const bool anyConnectedBefore = anySpotlightDeviceConnected();
-      connection.disable();
-      QTimer::singleShot(0, this, [this, devicePath=connection.path(), anyConnectedBefore](){
-        removeDeviceConnection(devicePath);
-        if (!anySpotlightDeviceConnected() && anyConnectedBefore) {
-          emit anySpotlightDeviceConnectedChanged(false);
+      // Logitech Spotlight:
+      //   * Next Button = 0xda
+      //   * Back Button = 0xdc
+      // Byte 5 and 7 indicate pressed buttons
+      // Back and next can be pressed at the same time
+
+      constexpr uint8_t ButtonNext = 0xda;
+      constexpr uint8_t ButtonBack = 0xdc;
+      const auto isNextPressed = msg[5] == ButtonNext || msg[7] == ButtonNext;
+      const auto isBackPressed = msg[5] == ButtonBack || msg[7] == ButtonBack;
+
+      if (!m_holdButtonStatus->nextPressed() && isNextPressed)
+      {
+        const auto& nextHold = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::NextHold);
+        for (const auto& ke: nextHold.keyEventSeq) {
+          connection->inputMapper()->addEvents(ke);
         }
-      });
-    }
-    return;
+      }
+
+      if (!m_holdButtonStatus->backPressed() && isBackPressed)
+      {
+        const auto& backHold = SpecialKeys::eventSequenceInfo(SpecialKeys::Key::BackHold);
+        for (const auto& ke: backHold.keyEventSeq) {
+          connection->inputMapper()->addEvents(ke);
+        }
+      }
+
+      m_holdButtonStatus->setButtonsPressed(isNextPressed, isBackPressed);
+    }), 0 /* function 0 */);
+
+    connection->registerNotificationCallback(this, rcIndex,
+    makeSafeCallback([this, connection](Message&& msg)
+    {
+      // Block some of the move events
+      // TODO This works quiet okay in combination with adjusting x and y values,
+      // but needs to be a more solid option to accumulate the mass of move events
+      // and consolidate them to a number of meaningful action special key events.
+      if (m_holdMoveEventTimer->isActive()) { return; }
+      m_holdMoveEventTimer->start();
+
+      // byte 4 : -1 for left movement, 0 for right movement
+      // byte 5 : horizontal movement speed -128 to 127
+      // byte 6 : -1 for up movement, 0 for down movement
+      // byte 7 : vertical movement speed -128 to 127
+
+      static const auto intcast = [](uint8_t v) -> int{ return static_cast<int8_t>(v); };
+
+      const int x = intcast(msg[5]);
+      const int y = intcast(msg[7]);
+
+      static const auto getReducedParam = [](int param) -> int {
+        constexpr int divider = 5;
+        constexpr int minimum = 5;
+        constexpr int maximum = 10;
+        if (std::abs(param) < minimum) { return 0; }
+        const auto sign = (param == 0) ? 0 : ((param > 0) ? 1 : -1);
+        return std::floor(1.0 * ((abs(param) > maximum)? sign * maximum : param) / divider);
+      };
+
+      const int adjustedX = getReducedParam(x);
+      const int adjustedY = getReducedParam(y);
+
+      if (adjustedX == 0 && adjustedY == 0) { return; }
+
+      static const auto scrollHAction = GlobalActions::scrollHorizontal();
+      scrollHAction->param = -adjustedX;
+
+      static const auto scrollVAction = GlobalActions::scrollVertical();
+      scrollVAction->param = adjustedY;
+
+      static const auto volumeControlAction = GlobalActions::volumeControl();
+      volumeControlAction->param = -adjustedY;
+
+      if (!connection->inputMapper()->recordingMode())
+      {
+          for (const auto& key_event : m_holdButtonStatus->moveKeyEventSeq()) {
+            connection->inputMapper()->addEvents(key_event);
+          }
+      }
+    }), 1 /* function 1 */);
   }
-  logDebug(hid) << "Received" << readVal.toHex() << "from" << connection.path();
-  // TODO: Process Logitech HIDPP message
 }
 
 // -------------------------------------------------------------------------------------------------
@@ -322,23 +553,7 @@ bool Spotlight::addInputEventHandler(std::shared_ptr<SubEventConnection> connect
   QSocketNotifier* const readNotifier = connection->socketReadNotifier();
   connect(readNotifier, &QSocketNotifier::activated, this,
   [this, connection=std::move(connection)](int fd) {
-    onEventDataAvailable(fd, *connection.get());
-  });
-
-  return true;
-}
-
-// -------------------------------------------------------------------------------------------------
-bool Spotlight::addHIDInputHandler(std::shared_ptr<SubHidrawConnection> connection)
-{
-  if (!connection || connection->type() != ConnectionType::Hidraw || !connection->isConnected()) {
-    return false;
-  }
-
-  QSocketNotifier* const readNotifier = connection->socketReadNotifier();
-  connect(readNotifier, &QSocketNotifier::activated, this,
-  [this, connection=std::move(connection)](int fd) {
-    onHIDDataAvailable(fd, *connection.get());
+    onEventDataAvailable(fd, *connection);
   });
 
   return true;
